@@ -280,7 +280,7 @@ gst_dvdemux_reset (GstDVDemux * dvdemux)
   dvdemux->audio_offset = 0;
   dvdemux->video_offset = 0;
   dvdemux->framecount = 0;
-  gst_atomic_int_set (&dvdemux->found_header, 0);
+  g_atomic_int_set (&dvdemux->found_header, 0);
   dvdemux->frame_len = -1;
   dvdemux->need_segment = FALSE;
   dvdemux->new_media = FALSE;
@@ -1449,7 +1449,7 @@ gst_dvdemux_flush (GstDVDemux * dvdemux)
       dvdemux->framerate_numerator = NTSC_FRAMERATE_NUMERATOR;
       dvdemux->framerate_denominator = NTSC_FRAMERATE_DENOMINATOR;
     }
-    gst_atomic_int_set (&dvdemux->found_header, 1);
+    g_atomic_int_set (&dvdemux->found_header, 1);
 
     /* let demux_video set the height, it needs to detect when things change so
      * it can reset caps */
@@ -1597,7 +1597,7 @@ gst_dvdemux_loop (GstPad * pad)
       /* setting header and prrforming the seek must be atomic */
       GST_OBJECT_LOCK (dvdemux);
       /* got header now */
-      gst_atomic_int_set (&dvdemux->found_header, 1);
+      g_atomic_int_set (&dvdemux->found_header, 1);
 
       /* now perform pending seek if any. */
       event = dvdemux->seek_event;
@@ -1681,10 +1681,10 @@ small_buffer:
   }
 pause:
   {
-    GST_LOG_OBJECT (dvdemux, "pausing task %d", ret);
+    GST_INFO_OBJECT (dvdemux, "pausing task, %s", gst_flow_get_name (ret));
     dvdemux->running = FALSE;
     gst_pad_pause_task (dvdemux->sinkpad);
-    if (GST_FLOW_IS_FATAL (ret)) {
+    if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
       if (ret == GST_FLOW_UNEXPECTED) {
         GST_LOG_OBJECT (dvdemux, "got eos");
         /* perform EOS logic */
@@ -1697,7 +1697,7 @@ pause:
           gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
         }
       } else {
-        /* for fatal errors we post an error message */
+        /* for fatal errors or not-linked we post an error message */
         GST_ELEMENT_ERROR (dvdemux, STREAM, FAILED,
             (NULL), ("streaming stopped, reason %s", gst_flow_get_name (ret)));
         gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
