@@ -240,6 +240,7 @@ gst_esdsink_open (GstAudioSink * asink)
 
   pad_template = gst_static_pad_template_get (&sink_factory);
   esdsink->cur_caps = gst_caps_copy (gst_pad_template_get_caps (pad_template));
+  gst_object_unref (pad_template);
 
   for (i = 0; i < esdsink->cur_caps->structs->len; i++) {
     GstStructure *s;
@@ -412,7 +413,7 @@ write_error:
   {
     GST_ELEMENT_ERROR (esdsink, RESOURCE, WRITE,
         ("Failed to write data to the esound daemon"), GST_ERROR_SYSTEM);
-    return 0;
+    return -1;
   }
 }
 
@@ -424,6 +425,11 @@ gst_esdsink_delay (GstAudioSink * asink)
 
   latency = esd_get_latency (esdsink->ctrl_fd);
 
+  if (latency == (guint) - 1) {
+    GST_WARNING_OBJECT (asink, "couldn't get latency");
+    return 0;
+  }
+
   /* latency is measured in samples at a rate of 44100, this 
    * cannot overflow. */
   latency = latency * G_GINT64_CONSTANT (44100) / esdsink->rate;
