@@ -89,6 +89,9 @@
 #endif
 
 #include <string.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>             /* atoi() */
+#endif
 #include <gst/gstelement.h>
 #include <gst/gst-i18n-plugin.h>
 #include <libsoup/soup.h>
@@ -132,6 +135,7 @@ static void gst_soup_http_src_uri_handler_init (gpointer g_iface,
 static void gst_soup_http_src_init (GstSoupHTTPSrc * src,
     GstSoupHTTPSrcClass * g_class);
 static void gst_soup_http_src_dispose (GObject * gobject);
+
 static void gst_soup_http_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_soup_http_src_get_property (GObject * object, guint prop_id,
@@ -140,12 +144,17 @@ static void gst_soup_http_src_get_property (GObject * object, guint prop_id,
 static GstFlowReturn gst_soup_http_src_create (GstPushSrc * psrc,
     GstBuffer ** outbuf);
 static gboolean gst_soup_http_src_start (GstBaseSrc * bsrc);
+
 static gboolean gst_soup_http_src_stop (GstBaseSrc * bsrc);
+
 static gboolean gst_soup_http_src_get_size (GstBaseSrc * bsrc, guint64 * size);
+
 static gboolean gst_soup_http_src_is_seekable (GstBaseSrc * bsrc);
+
 static gboolean gst_soup_http_src_do_seek (GstBaseSrc * bsrc,
     GstSegment * segment);
 static gboolean gst_soup_http_src_unlock (GstBaseSrc * bsrc);
+
 static gboolean gst_soup_http_src_unlock_stop (GstBaseSrc * bsrc);
 
 static gboolean gst_soup_http_src_set_location (GstSoupHTTPSrc * src,
@@ -156,16 +165,23 @@ static gboolean gst_soup_http_src_set_proxy (GstSoupHTTPSrc * src,
 static char *gst_soup_http_src_unicodify (const char *str);
 
 static gboolean gst_soup_http_src_build_message (GstSoupHTTPSrc * src);
+
 static void gst_soup_http_src_cancel_message (GstSoupHTTPSrc * src);
+
 static void gst_soup_http_src_queue_message (GstSoupHTTPSrc * src);
+
 static gboolean gst_soup_http_src_add_range_header (GstSoupHTTPSrc * src,
     guint64 offset);
 static void gst_soup_http_src_session_unpause_message (GstSoupHTTPSrc * src);
+
 static void gst_soup_http_src_session_pause_message (GstSoupHTTPSrc * src);
+
 static void gst_soup_http_src_session_close (GstSoupHTTPSrc * src);
+
 static void gst_soup_http_src_parse_status (SoupMessage * msg,
     GstSoupHTTPSrc * src);
 static void gst_soup_http_src_chunk_free (gpointer gstbuf);
+
 static SoupBuffer *gst_soup_http_src_chunk_allocator (SoupMessage * msg,
     gsize max_len, gpointer user_data);
 static void gst_soup_http_src_got_chunk_cb (SoupMessage * msg,
@@ -213,7 +229,9 @@ static void
 gst_soup_http_src_class_init (GstSoupHTTPSrcClass * klass)
 {
   GObjectClass *gobject_class;
+
   GstBaseSrcClass *gstbasesrc_class;
+
   GstPushSrcClass *gstpushsrc_class;
 
   gobject_class = (GObjectClass *) klass;
@@ -437,7 +455,7 @@ gst_soup_http_src_get_property (GObject * object, guint prop_id,
         char *proxy = soup_uri_to_string (src->proxy, FALSE);
 
         g_value_set_string (value, proxy);
-        free (proxy);
+        g_free (proxy);
       }
       break;
     case PROP_COOKIES:
@@ -477,8 +495,10 @@ gst_soup_http_src_unicodify (const gchar * str)
 static void
 gst_soup_http_src_cancel_message (GstSoupHTTPSrc * src)
 {
-  if (src->msg != NULL)
+  if (src->msg != NULL) {
+    src->session_io_status = GST_SOUP_HTTP_SRC_SESSION_IO_STATUS_CANCELLED;
     soup_session_cancel_message (src->session, src->msg, SOUP_STATUS_CANCELLED);
+  }
   src->session_io_status = GST_SOUP_HTTP_SRC_SESSION_IO_STATUS_IDLE;
   src->msg = NULL;
 }
@@ -495,6 +515,7 @@ static gboolean
 gst_soup_http_src_add_range_header (GstSoupHTTPSrc * src, guint64 offset)
 {
   gchar buf[64];
+
   gint rc;
 
   soup_message_headers_remove (src->msg->request_headers, "Range");
@@ -535,8 +556,11 @@ static void
 gst_soup_http_src_got_headers_cb (SoupMessage * msg, GstSoupHTTPSrc * src)
 {
   const char *value;
+
   GstTagList *tag_list;
+
   GstBaseSrc *basesrc;
+
   guint64 newsize;
 
   GST_DEBUG_OBJECT (src, "got headers");
@@ -547,6 +571,9 @@ gst_soup_http_src_got_headers_cb (SoupMessage * msg, GstSoupHTTPSrc * src)
     return;
   }
 
+  if (msg->status_code == SOUP_STATUS_UNAUTHORIZED)
+    return;
+
   src->session_io_status = GST_SOUP_HTTP_SRC_SESSION_IO_STATUS_RUNNING;
 
   /* Parse Content-Length. */
@@ -670,8 +697,12 @@ gst_soup_http_src_finished_cb (SoupMessage * msg, GstSoupHTTPSrc * src)
   }
   GST_DEBUG_OBJECT (src, "finished");
   src->ret = GST_FLOW_UNEXPECTED;
-  if (src->session_io_status == GST_SOUP_HTTP_SRC_SESSION_IO_STATUS_RUNNING &&
-      src->read_position > 0) {
+  if (src->session_io_status == GST_SOUP_HTTP_SRC_SESSION_IO_STATUS_CANCELLED) {
+    /* gst_soup_http_src_cancel_message() triggered this; probably a seek
+     * that occurred in the QUEUEING state; i.e. before the connection setup
+     * was complete. Do nothing */
+  } else if (src->session_io_status ==
+      GST_SOUP_HTTP_SRC_SESSION_IO_STATUS_RUNNING && src->read_position > 0) {
     /* The server disconnected while streaming. Reconnect and seeking to the
      * last location. */
     src->retry = TRUE;
@@ -720,10 +751,15 @@ gst_soup_http_src_chunk_allocator (SoupMessage * msg, gsize max_len,
     gpointer user_data)
 {
   GstSoupHTTPSrc *src = (GstSoupHTTPSrc *) user_data;
+
   GstBaseSrc *basesrc = GST_BASE_SRC_CAST (src);
+
   GstBuffer *gstbuf;
+
   SoupBuffer *soupbuf;
+
   gsize length;
+
   GstFlowReturn rc;
 
   if (max_len)
@@ -757,6 +793,7 @@ gst_soup_http_src_got_chunk_cb (SoupMessage * msg, SoupBuffer * chunk,
     GstSoupHTTPSrc * src)
 {
   GstBaseSrc *basesrc;
+
   guint64 new_position;
 
   if (G_UNLIKELY (msg != src->msg)) {
@@ -779,8 +816,9 @@ gst_soup_http_src_got_chunk_cb (SoupMessage * msg, SoupBuffer * chunk,
   GST_BUFFER_OFFSET (*src->outbuf) = basesrc->segment.last_stop;
 
   gst_buffer_set_caps (*src->outbuf,
-      (src->icy_caps) ? src->
-      icy_caps : GST_PAD_CAPS (GST_BASE_SRC_PAD (basesrc)));
+      (src->
+          icy_caps) ? src->icy_caps :
+      GST_PAD_CAPS (GST_BASE_SRC_PAD (basesrc)));
 
   new_position = src->read_position + chunk->length;
   if (G_LIKELY (src->request_position == src->read_position))
@@ -816,6 +854,8 @@ gst_soup_http_src_response_cb (SoupSession * session, SoupMessage * msg,
     src->retry = TRUE;
   } else
     gst_soup_http_src_parse_status (msg, src);
+  /* The session's SoupMessage object expires after this callback returns. */
+  src->msg = NULL;
   g_main_loop_quit (src->loop);
 }
 
@@ -890,6 +930,7 @@ gst_soup_http_src_build_message (GstSoupHTTPSrc * src)
           *cookie);
     }
   }
+  src->retry = FALSE;
 
   g_signal_connect (src->msg, "got_headers",
       G_CALLBACK (gst_soup_http_src_got_headers_cb), src);
@@ -933,7 +974,7 @@ gst_soup_http_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
   src->outbuf = outbuf;
   do {
     if (src->interrupted) {
-      gst_soup_http_src_cancel_message (src);
+      GST_DEBUG_OBJECT (src, "interrupted");
       break;
     }
     if (src->retry) {
@@ -958,6 +999,9 @@ gst_soup_http_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
       case GST_SOUP_HTTP_SRC_SESSION_IO_STATUS_RUNNING:
         gst_soup_http_src_session_unpause_message (src);
         break;
+      case GST_SOUP_HTTP_SRC_SESSION_IO_STATUS_CANCELLED:
+        /* Impossible. */
+        break;
     }
 
     if (src->ret == GST_FLOW_CUSTOM_ERROR)
