@@ -106,6 +106,9 @@ static void gst_speex_dec_get_property (GObject * object, guint prop_id,
 static void gst_speex_dec_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 
+static GstFlowReturn speex_dec_chain_parse_data (GstSpeexDec * dec,
+    GstBuffer * buf, GstClockTime timestamp, GstClockTime duration);
+
 static void
 gst_speex_dec_base_init (gpointer g_class)
 {
@@ -457,6 +460,18 @@ speex_dec_sink_event (GstPad * pad, GstEvent * event)
       if (rate <= 0.0)
         goto newseg_wrong_rate;
 
+      if (update) {
+        /* time progressed without data, see if we can fill the gap with
+         * some concealment data */
+        if (dec->segment.last_stop < start) {
+          GstClockTime duration;
+
+          duration = start - dec->segment.last_stop;
+          speex_dec_chain_parse_data (dec, NULL, dec->segment.last_stop,
+              duration);
+        }
+      }
+
       /* now configure the values */
       gst_segment_set_newsegment_full (&dec->segment, update,
           rate, arate, GST_FORMAT_TIME, start, stop, time);
@@ -623,49 +638,57 @@ speex_dec_chain_parse_comments (GstSpeexDec * dec, GstBuffer * buf)
 }
 
 static GstFlowReturn
-speex_dec_chain_parse_data (GstSpeexDec * dec, GstBuffer * buf)
+speex_dec_chain_parse_data (GstSpeexDec * dec, GstBuffer * buf,
+    GstClockTime timestamp, GstClockTime duration)
 {
   GstFlowReturn res = GST_FLOW_OK;
   gint i, fpp;
   guint size;
   guint8 *data;
+  SpeexBits *bits;
 
-  data = GST_BUFFER_DATA (buf);
-  size = GST_BUFFER_SIZE (buf);
+  if (timestamp != -1) {
+    dec->segment.last_stop = timestamp;
+    dec->granulepos = -1;
+  }
 
-  /* send data to the bitstream */
-  speex_bits_read_from (&dec->bits, (char *) data, size);
+  if (buf) {
+    data = GST_BUFFER_DATA (buf);
+    size = GST_BUFFER_SIZE (buf);
 
-  fpp = dec->header->frames_per_packet;
+    /* send data to the bitstream */
+    speex_bits_read_from (&dec->bits, (char *) data, size);
+
+    fpp = dec->header->frames_per_packet;
+    bits = &dec->bits;
+
+    GST_DEBUG_OBJECT (dec, "received buffer of size %u, fpp %d", size, fpp);
+
+    if (!GST_BUFFER_TIMESTAMP_IS_VALID (buf)
+        && GST_BUFFER_OFFSET_END_IS_VALID (buf)) {
+      dec->granulepos = GST_BUFFER_OFFSET_END (buf);
+      GST_DEBUG_OBJECT (dec,
+          "Taking granulepos from upstream: %" G_GUINT64_FORMAT,
+          dec->granulepos);
+    }
+
+    /* copy timestamp */
+  } else {
+    /* concealment data, pass NULL as the bits parameters */
+    GST_DEBUG_OBJECT (dec, "creating concealment data");
+    fpp = dec->header->frames_per_packet;
+    bits = NULL;
+  }
 
-  GST_DEBUG_OBJECT (dec, "received buffer of size %u, fpp %d", size, fpp);
 
   /* now decode each frame */
   for (i = 0; i < fpp; i++) {
     GstBuffer *outbuf;
-    gint64 timestamp;
     gint16 *out_data;
-    gint ret, j;
+    gint ret;
 
     GST_LOG_OBJECT (dec, "decoding frame %d/%d", i, fpp);
 
-    ret = speex_decode (dec->state, &dec->bits, dec->output);
-    if (ret == -1) {
-      /* uh? end of stream */
-      GST_WARNING_OBJECT (dec, "Unexpected end of stream found");
-      break;
-    } else if (ret == -2) {
-      GST_WARNING_OBJECT (dec, "Decoding error: corrupted stream?");
-      break;
-    }
-
-    if (speex_bits_remaining (&dec->bits) < 0) {
-      GST_WARNING_OBJECT (dec, "Decoding overflow: corrupted stream?");
-      break;
-    }
-    if (dec->header->nb_channels == 2)
-      speex_decode_stereo (dec->output, dec->frame_size, &dec->stereo);
-
     res = gst_pad_alloc_buffer_and_set_caps (dec->srcpad,
         GST_BUFFER_OFFSET_NONE, dec->frame_size * dec->header->nb_channels * 2,
         GST_PAD_CAPS (dec->srcpad), &outbuf);
@@ -677,32 +700,47 @@ speex_dec_chain_parse_data (GstSpeexDec * dec, GstBuffer * buf)
 
     out_data = (gint16 *) GST_BUFFER_DATA (outbuf);
 
-    /*PCM saturation (just in case) */
-    for (j = 0; j < dec->frame_size * dec->header->nb_channels; j++) {
-      if (dec->output[j] > 32767.0)
-        out_data[j] = 32767;
-      else if (dec->output[i] < -32768.0)
-        out_data[j] = -32768;
-      else
-        out_data[j] = (gint16) dec->output[j];
+    ret = speex_decode_int (dec->state, bits, out_data);
+    if (ret == -1) {
+      /* uh? end of stream */
+      GST_WARNING_OBJECT (dec, "Unexpected end of stream found");
+      gst_buffer_unref (outbuf);
+      outbuf = NULL;
+      break;
+    } else if (ret == -2) {
+      GST_WARNING_OBJECT (dec, "Decoding error: corrupted stream?");
+      gst_buffer_unref (outbuf);
+      outbuf = NULL;
+      break;
     }
 
+    if (bits && speex_bits_remaining (bits) < 0) {
+      GST_WARNING_OBJECT (dec, "Decoding overflow: corrupted stream?");
+      gst_buffer_unref (outbuf);
+      outbuf = NULL;
+      break;
+    }
+    if (dec->header->nb_channels == 2)
+      speex_decode_stereo_int (out_data, dec->frame_size, &dec->stereo);
+
     if (dec->granulepos == -1) {
       if (dec->segment.format != GST_FORMAT_TIME) {
         GST_WARNING_OBJECT (dec, "segment not initialized or not TIME format");
-        dec->granulepos = 0;
+        dec->granulepos = dec->frame_size;
       } else {
         dec->granulepos = gst_util_uint64_scale_int (dec->segment.last_stop,
-            dec->header->rate, GST_SECOND);
+            dec->header->rate, GST_SECOND) + dec->frame_size;
       }
       GST_DEBUG_OBJECT (dec, "granulepos=%" G_GINT64_FORMAT, dec->granulepos);
     }
 
-    timestamp = gst_util_uint64_scale_int (dec->granulepos,
-        GST_SECOND, dec->header->rate);
+    if (timestamp == -1) {
+      timestamp = gst_util_uint64_scale_int (dec->granulepos - dec->frame_size,
+          GST_SECOND, dec->header->rate);
+    }
 
-    GST_BUFFER_OFFSET (outbuf) = dec->granulepos;
-    GST_BUFFER_OFFSET_END (outbuf) = dec->granulepos + dec->frame_size;
+    GST_BUFFER_OFFSET (outbuf) = dec->granulepos - dec->frame_size;
+    GST_BUFFER_OFFSET_END (outbuf) = dec->granulepos;
     GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
     GST_BUFFER_DURATION (outbuf) = dec->frame_duration;
 
@@ -719,6 +757,7 @@ speex_dec_chain_parse_data (GstSpeexDec * dec, GstBuffer * buf)
       GST_DEBUG_OBJECT (dec, "flow: %s", gst_flow_get_name (res));
       break;
     }
+    timestamp = -1;
   }
 
   return res;
@@ -740,7 +779,9 @@ speex_dec_chain (GstPad * pad, GstBuffer * buf)
       res = speex_dec_chain_parse_comments (dec, buf);
       break;
     default:
-      res = speex_dec_chain_parse_data (dec, buf);
+      res =
+          speex_dec_chain_parse_data (dec, buf, GST_BUFFER_TIMESTAMP (buf),
+          GST_BUFFER_DURATION (buf));
       break;
   }
 
