@@ -109,17 +109,19 @@ gst_ape_demux_class_init (GstApeDemuxClass * klass)
 
   tagdemux_class->identify_tag = GST_DEBUG_FUNCPTR (gst_ape_demux_identify_tag);
   tagdemux_class->parse_tag = GST_DEBUG_FUNCPTR (gst_ape_demux_parse_tag);
+
+  /* no need for a merge function, the default behaviour to prefer start
+   * tags (APEv2) over end tags (usually APEv1, but could theoretically also
+   * be APEv2) is fine */
+
+  tagdemux_class->min_start_size = 32;
+  tagdemux_class->min_end_size = 32;
 }
 
 static void
 gst_ape_demux_init (GstApeDemux * apedemux, GstApeDemuxClass * gclass)
 {
-  GstTagDemux *tagdemux = GST_TAG_DEMUX (apedemux);
-
-  tagdemux->min_start_size = 32;
-  tagdemux->min_end_size = 32;
-
-  tagdemux->prefer_start_tag = TRUE;
+  /* nothing to do here */
 }
 
 static const struct _GstApeDemuxTagTableEntry
@@ -135,12 +137,18 @@ static const struct _GstApeDemuxTagTableEntry
   "title", GST_TAG_TITLE}, {
   "artist", GST_TAG_ARTIST}, {
   "album", GST_TAG_ALBUM}, {
+  "composer", GST_TAG_COMPOSER}, {
   "comment", GST_TAG_COMMENT}, {
   "comments", GST_TAG_COMMENT}, {
   "copyright", GST_TAG_COPYRIGHT}, {
   "genre", GST_TAG_GENRE}, {
   "isrc", GST_TAG_ISRC}, {
+  "disc", GST_TAG_ALBUM_VOLUME_NUMBER}, {
+  "disk", GST_TAG_ALBUM_VOLUME_NUMBER}, {
+  "discnumber", GST_TAG_ALBUM_VOLUME_NUMBER}, {
+  "disknumber", GST_TAG_ALBUM_VOLUME_NUMBER}, {
   "track", GST_TAG_TRACK_NUMBER}, {
+  "tracknumber", GST_TAG_TRACK_NUMBER}, {
   "year", GST_TAG_DATE}, {
   "file", GST_TAG_LOCATION}
 };
@@ -203,6 +211,21 @@ ape_demux_parse_tags (const guint8 * data, gint size)
     tag = g_strndup ((gchar *) data + 8, n - 9);
     val = g_strndup ((gchar *) data + n, len);
 
+    /* special-case 'media' tag, could be e.g. "CD 1/2" */
+    if (g_ascii_strcasecmp (tag, "media") == 0) {
+      gchar *sp, *sp2;
+
+      g_free (tag);
+      tag = g_strdup ("discnumber");
+      /* get rid of the medium in front */
+      sp = strchr (val, ' ');
+      while (sp != NULL && (sp2 = strchr (sp + 1, ' ')) != NULL)
+        sp = sp2;
+      if (sp) {
+        g_memmove (val, sp + 1, strlen (sp + 1) + 1);
+      }
+    }
+
     if (ape_demux_get_gst_tag_from_tag (tag, &gst_tag, &gst_tag_type)) {
       GValue v = { 0, };
 
@@ -226,12 +249,25 @@ ape_demux_parse_tags (const guint8 * data, gint size)
               g_value_init (&v, G_TYPE_UINT);
               g_value_set_uint (&v, v_uint);
             }
-            GST_LOG ("checking for count: %s", val);
+            GST_LOG ("checking for track count: %s", val);
             /* might be 0/N or -1/N to specify that there is only a count */
             if (sscanf (val, "%d/%u", &dummy, &count) == 2 && count > 0) {
               gst_tag_list_add (taglist, GST_TAG_MERGE_APPEND,
                   GST_TAG_TRACK_COUNT, count, NULL);
             }
+          } else if (strcmp (gst_tag, GST_TAG_ALBUM_VOLUME_NUMBER) == 0) {
+            gint dummy;
+
+            if (sscanf (val, "%u", &v_uint) == 1 && v_uint > 0) {
+              g_value_init (&v, G_TYPE_UINT);
+              g_value_set_uint (&v, v_uint);
+            }
+            GST_LOG ("checking for volume count: %s", val);
+            /* might be 0/N or -1/N to specify that there is only a count */
+            if (sscanf (val, "%d/%u", &dummy, &count) == 2 && count > 0) {
+              gst_tag_list_add (taglist, GST_TAG_MERGE_APPEND,
+                  GST_TAG_ALBUM_VOLUME_COUNT, count, NULL);
+            }
           } else if (sscanf (val, "%u", &v_uint) == 1) {
             g_value_init (&v, G_TYPE_UINT);
             g_value_set_uint (&v, v_uint);
@@ -245,11 +281,18 @@ ape_demux_parse_tags (const guint8 * data, gint size)
         }
         case G_TYPE_DOUBLE:{
           gdouble v_double;
+          gchar *endptr;
 
-          if (sscanf (val, "%lf", &v_double) == 1) {
+          /* floating point strings can be "4,123" or "4.123" depending on
+           * the locale. We need to be able to parse and read either version
+           * no matter what our current locale is */
+          g_strdelimit (val, ",", '.');
+          v_double = g_ascii_strtod (val, &endptr);
+          if (endptr != val) {
             g_value_init (&v, G_TYPE_DOUBLE);
             g_value_set_double (&v, v_double);
           }
+
           break;
         }
         default:{
