@@ -182,10 +182,13 @@ static GstStaticPadTemplate audio_sink_factory =
         "rate = (int) [ 1000, 96000 ], " "channels = (int) [ 1, 2 ]; "
 #endif*/
         "audio/x-ac3, "
-        "rate = (int) [ 1000, 96000 ], " "channels = (int) [ 1, 2 ]")
+        "rate = (int) [ 1000, 96000 ], " "channels = (int) [ 1, 2 ]; "
+        "audio/x-alaw, "
+        "rate = (int) [ 1000, 48000 ], " "channels = (int) [ 1, 2 ]; "
+        "audio/x-mulaw, "
+        "rate = (int) [ 1000, 48000 ], " "channels = (int) [ 1, 2 ]; ")
     );
 
-
 static void gst_avi_mux_base_init (gpointer g_class);
 static void gst_avi_mux_class_init (GstAviMuxClass * klass);
 static void gst_avi_mux_init (GstAviMux * avimux);
@@ -314,8 +317,6 @@ gst_avi_mux_pad_reset (GstAviPad * avipad, gboolean free)
 {
   /* generic part */
   memset (&(avipad->hdr), 0, sizeof (gst_riff_strh));
-  avipad->hdr.rate = 1;
-  avipad->hdr.quality = 0xFFFFFFFF;
 
   memset (&(avipad->idx[0]), 0, sizeof (avipad->idx));
 
@@ -329,15 +330,18 @@ gst_avi_mux_pad_reset (GstAviPad * avipad, gboolean free)
   if (avipad->is_video) {
     GstAviVideoPad *vidpad = (GstAviVideoPad *) avipad;
 
+    avipad->hdr.type = GST_MAKE_FOURCC ('v', 'i', 'd', 's');
     if (vidpad->vids_codec_data) {
       gst_buffer_unref (vidpad->vids_codec_data);
       vidpad->vids_codec_data = NULL;
     }
 
     memset (&(vidpad->vids), 0, sizeof (gst_riff_strf_vids));
+    memset (&(vidpad->vprp), 0, sizeof (gst_riff_vprp));
   } else {
     GstAviAudioPad *audpad = (GstAviAudioPad *) avipad;
 
+    avipad->hdr.type = GST_MAKE_FOURCC ('a', 'u', 'd', 's');
     if (audpad->auds_codec_data) {
       gst_buffer_unref (audpad->auds_codec_data);
       audpad->auds_codec_data = NULL;
@@ -422,9 +426,10 @@ gst_avi_mux_vidsink_set_caps (GstPad * pad, GstCaps * vscaps)
   GstAviCollectData *collect_pad;
   GstStructure *structure;
   const gchar *mimetype;
-  const GValue *fps;
+  const GValue *fps, *par;
   const GValue *codec_data;
   gint width, height;
+  gint par_n, par_d;
 
   avimux = GST_AVI_MUX (gst_pad_get_parent (pad));
 
@@ -460,6 +465,36 @@ gst_avi_mux_vidsink_set_caps (GstPad * pad, GstCaps * vscaps)
   avipad->parent.hdr.rate = gst_value_get_fraction_numerator (fps);
   avipad->parent.hdr.scale = gst_value_get_fraction_denominator (fps);
 
+  /* (pixel) aspect ratio data, if any */
+  par = gst_structure_get_value (structure, "pixel-aspect-ratio");
+  /* only use video properties header if there is non-trivial aspect info */
+  if (par && GST_VALUE_HOLDS_FRACTION (par) &&
+      ((par_n = gst_value_get_fraction_numerator (par)) !=
+          (par_d = gst_value_get_fraction_denominator (par)))) {
+    GValue to_ratio = { 0, };
+    guint ratio_n, ratio_d;
+
+    /* some fraction voodoo to obtain simplest possible ratio */
+    g_value_init (&to_ratio, GST_TYPE_FRACTION);
+    gst_value_set_fraction (&to_ratio, width * par_n, height * par_d);
+    ratio_n = gst_value_get_fraction_numerator (&to_ratio);
+    ratio_d = gst_value_get_fraction_denominator (&to_ratio);
+    GST_DEBUG_OBJECT (avimux, "generating vprp data with aspect ratio %d/%d",
+        ratio_n, ratio_d);
+    /* simply fill in */
+    avipad->vprp.vert_rate = avipad->parent.hdr.rate / avipad->parent.hdr.scale;
+    avipad->vprp.hor_t_total = width;
+    avipad->vprp.vert_lines = height;
+    avipad->vprp.aspect = (ratio_n) << 16 | (ratio_d & 0xffff);
+    avipad->vprp.width = width;
+    avipad->vprp.height = height;
+    avipad->vprp.fields = 1;
+    avipad->vprp.field_info[0].compressed_bm_height = height;
+    avipad->vprp.field_info[0].compressed_bm_width = width;
+    avipad->vprp.field_info[0].valid_bm_height = height;
+    avipad->vprp.field_info[0].valid_bm_width = width;
+  }
+
   /* codec initialization data, if any */
   codec_data = gst_structure_get_value (structure, "codec_data");
   if (codec_data) {
@@ -629,11 +664,28 @@ gst_avi_mux_audsink_set_caps (GstPad * pad, GstCaps * vscaps)
 
   if (!strcmp (mimetype, "audio/x-raw-int")) {
     gint width, depth;
+    gboolean signedness;
 
     avipad->auds.format = GST_RIFF_WAVE_FORMAT_PCM;
 
     if (!gst_structure_get_int (structure, "width", &width) ||
-        (width != 8 && !gst_structure_get_int (structure, "depth", &depth))) {
+        !gst_structure_get_int (structure, "depth", &depth) ||
+        !gst_structure_get_boolean (structure, "signed", &signedness)) {
+      GST_DEBUG_OBJECT (avimux,
+          "broken caps, width/depth/signed field missing");
+      goto refuse_caps;
+    }
+
+    /* no clear place to put different values for these while keeping to spec */
+    if (width != depth) {
+      GST_DEBUG_OBJECT (avimux, "width must be same as depth!");
+      goto refuse_caps;
+    }
+
+    /* because that's the way the caps will be recreated from riff data */
+    if ((width == 8 && signedness) || (width == 16 && !signedness)) {
+      GST_DEBUG_OBJECT (avimux,
+          "8-bit PCM must be unsigned, 16-bit PCM signed");
       goto refuse_caps;
     }
 
@@ -644,10 +696,12 @@ gst_avi_mux_audsink_set_caps (GstPad * pad, GstCaps * vscaps)
     avipad->auds.blockalign /= 8;
     avipad->auds.blockalign *= avipad->auds.channels;
     avipad->auds.av_bps = avipad->auds.blockalign * avipad->auds.rate;
-  } else if (!strcmp (mimetype, "audio/mpeg") ||
-      !strcmp (mimetype, "audio/x-vorbis") ||
-      !strcmp (mimetype, "audio/x-ac3")) {
+  } else {
     avipad->auds.format = 0;
+    /* set some defaults */
+    avipad->auds.blockalign = 1;
+    avipad->auds.av_bps = 0;
+    avipad->auds.size = 16;
 
     if (!strcmp (mimetype, "audio/mpeg")) {
       gint mpegversion;
@@ -678,17 +732,24 @@ gst_avi_mux_audsink_set_caps (GstPad * pad, GstCaps * vscaps)
       avipad->auds.format = GST_RIFF_WAVE_FORMAT_VORBIS3;
     } else if (!strcmp (mimetype, "audio/x-ac3")) {
       avipad->auds.format = GST_RIFF_WAVE_FORMAT_A52;
+    } else if (!strcmp (mimetype, "audio/x-alaw")) {
+      avipad->auds.format = GST_RIFF_WAVE_FORMAT_ALAW;
+      avipad->auds.size = 8;
+      avipad->auds.blockalign = avipad->auds.channels;
+      avipad->auds.av_bps = avipad->auds.blockalign * avipad->auds.rate;
+    } else if (!strcmp (mimetype, "audio/x-mulaw")) {
+      avipad->auds.format = GST_RIFF_WAVE_FORMAT_MULAW;
+      avipad->auds.av_bps = 8;
+      avipad->auds.size = 8;
+      avipad->auds.blockalign = avipad->auds.channels;
+      avipad->auds.av_bps = avipad->auds.blockalign * avipad->auds.rate;
     }
-
-    avipad->auds.blockalign = 1;
-    avipad->auds.av_bps = 0;
-    avipad->auds.size = 16;
-
-    if (!avipad->auds.format)
-      goto refuse_caps;
   }
 
-  avipad->parent.hdr.rate = avipad->auds.blockalign * avipad->auds.rate;
+  if (!avipad->auds.format)
+    goto refuse_caps;
+
+  avipad->parent.hdr.rate = avipad->auds.rate;
   avipad->parent.hdr.samplesize = avipad->auds.blockalign;
   avipad->parent.hdr.scale = 1;
 
@@ -711,29 +772,27 @@ gst_avi_mux_request_new_pad (GstElement * element,
   GstAviMux *avimux;
   GstPad *newpad;
   GstAviPad *avipad;
-  GstElementClass *klass = GST_ELEMENT_GET_CLASS (element);
+  GstElementClass *klass;
 
   g_return_val_if_fail (templ != NULL, NULL);
 
-  if (templ->direction != GST_PAD_SINK) {
-    g_warning ("avimux: request pad that is not a SINK pad\n");
-    return NULL;
-  }
+  if (templ->direction != GST_PAD_SINK)
+    goto wrong_direction;
 
   g_return_val_if_fail (GST_IS_AVI_MUX (element), NULL);
-
   avimux = GST_AVI_MUX (element);
 
-  if (!avimux->write_header) {
-    g_warning ("avimux: request pad cannot be added after streaming started\n");
-    return NULL;
-  }
+  if (!avimux->write_header)
+    goto too_late;
+
+  klass = GST_ELEMENT_GET_CLASS (element);
 
   if (templ == gst_element_class_get_pad_template (klass, "audio_%d")) {
     gchar *name;
 
     /* setup pad */
     name = g_strdup_printf ("audio_%02d", avimux->audio_pads);
+    GST_DEBUG_OBJECT (avimux, "adding new pad: %s", name);
     newpad = gst_pad_new_from_template (templ, name);
     g_free (name);
     gst_pad_set_setcaps_function (newpad,
@@ -753,6 +812,7 @@ gst_avi_mux_request_new_pad (GstElement * element,
     if (avimux->video_pads > 0)
       return NULL;
     /* setup pad */
+    GST_DEBUG_OBJECT (avimux, "adding new pad: video_00");
     newpad = gst_pad_new_from_template (templ, "video_00");
     gst_pad_set_setcaps_function (newpad,
         GST_DEBUG_FUNCPTR (gst_avi_mux_vidsink_set_caps));
@@ -764,10 +824,8 @@ gst_avi_mux_request_new_pad (GstElement * element,
     avimux->video_pads++;
     /* video goes first */
     avimux->sinkpads = g_slist_prepend (avimux->sinkpads, avipad);
-  } else {
-    g_warning ("avimux: this is not our template!\n");
-    return NULL;
-  }
+  } else
+    goto wrong_template;
 
   avipad->collect = gst_collect_pads_add_pad (avimux->collect,
       newpad, sizeof (GstAviCollectData));
@@ -782,6 +840,23 @@ gst_avi_mux_request_new_pad (GstElement * element,
   gst_element_add_pad (element, newpad);
 
   return newpad;
+
+  /* ERRORS */
+wrong_direction:
+  {
+    g_warning ("avimux: request pad that is not a SINK pad\n");
+    return NULL;
+  }
+too_late:
+  {
+    g_warning ("avimux: request pad cannot be added after streaming started\n");
+    return NULL;
+  }
+wrong_template:
+  {
+    g_warning ("avimuxx: this is not our template!\n");
+    return NULL;
+  }
 }
 
 static void
@@ -897,29 +972,26 @@ gst_avi_mux_riff_get_avi_header (GstAviMux * avimux)
   /* need to take snapshot of tags now */
   iface_tags = gst_tag_setter_get_tag_list (GST_TAG_SETTER (avimux));
   if ((iface_tags || avimux->tags) && !avimux->tags_snap) {
-    if (iface_tags && avimux->tags) {
-      tags = gst_tag_list_merge (iface_tags, avimux->tags,
-          GST_TAG_MERGE_APPEND);
-    } else if (iface_tags) {
-      tags = gst_tag_list_copy (iface_tags);
-    } else {
-      tags = gst_tag_list_copy (avimux->tags);
-    }
+    /* gst_tag_list_merge() will handle NULL for either or both lists fine */
+    tags = gst_tag_list_merge (iface_tags, avimux->tags,
+        gst_tag_setter_get_tag_merge_mode (GST_TAG_SETTER (avimux)));
     gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_ENCODER,
         PACKAGE_STRING " AVI muxer", NULL);
   } else {
     tags = avimux->tags_snap;
   }
   avimux->tags_snap = tags;
+  /* FIXME: that should be the strlen of all tags + header sizes */
   if (avimux->tags_snap)
     size += 1024;
 
   /* allocate the buffer, starting with some wild/safe upper bound */
   size += avimux->codec_data_size + 100 + sizeof (gst_riff_avih)
-      + (g_slist_length (avimux->sinkpads) * (100 + sizeof (gst_riff_strh)
+      + (g_slist_length (avimux->sinkpads) * (100 + sizeof (gst_riff_strh_full)
           + sizeof (gst_riff_strf_vids)
-          + sizeof (gst_riff_strf_auds)
-          + ODML_SUPERINDEX_SIZE));
+          + sizeof (gst_riff_vprp)
+          + sizeof (gst_riff_vprp_video_field_desc) * 2
+          + sizeof (gst_riff_strf_auds) + 2 + ODML_SUPERINDEX_SIZE));
   buffer = gst_buffer_new_and_alloc (size);
   buffdata = GST_BUFFER_DATA (buffer);
   highmark = 0;
@@ -964,18 +1036,26 @@ gst_avi_mux_riff_get_avi_header (GstAviMux * avimux)
     GstAviPad *avipad = (GstAviPad *) node->data;
     GstAviVideoPad *vidpad = (GstAviVideoPad *) avipad;
     GstAviAudioPad *audpad = (GstAviAudioPad *) avipad;
-    guint codec_size = 0, strl_size = 0;
+    guint codec_size = 0, strl_size = 0, vprp_size = 0;
 
     if (avipad->is_video) {
       if (vidpad->vids_codec_data)
         codec_size = GST_BUFFER_SIZE (vidpad->vids_codec_data);
-      strl_size = sizeof (gst_riff_strh) + sizeof (gst_riff_strf_vids)
-          + codec_size + 4 * 5 + ODML_SUPERINDEX_SIZE;
+      strl_size = sizeof (gst_riff_strh_full) + sizeof (gst_riff_strf_vids)
+          + GST_ROUND_UP_2 (codec_size) + 4 * 5 + ODML_SUPERINDEX_SIZE;
+      if (vidpad->vprp.aspect) {
+        /* let's be on the safe side */
+        vidpad->vprp.fields = MIN (vidpad->vprp.fields,
+            GST_RIFF_VPRP_VIDEO_FIELDS);
+        vprp_size = G_STRUCT_OFFSET (gst_riff_vprp, field_info)
+            + (vidpad->vprp.fields * sizeof (gst_riff_vprp_video_field_desc));
+        strl_size += 4 * 2 + vprp_size;
+      }
     } else {
       if (audpad->auds_codec_data)
         codec_size = GST_BUFFER_SIZE (audpad->auds_codec_data);
-      strl_size = sizeof (gst_riff_strh) + sizeof (gst_riff_strf_auds)
-          + codec_size + 4 * 5 + ODML_SUPERINDEX_SIZE;
+      strl_size = sizeof (gst_riff_strh_full) + sizeof (gst_riff_strf_auds) + 2
+          + GST_ROUND_UP_2 (codec_size) + 4 * 5 + ODML_SUPERINDEX_SIZE;
     }
 
     /* stream list metadata */
@@ -984,7 +1064,7 @@ gst_avi_mux_riff_get_avi_header (GstAviMux * avimux)
     memcpy (buffdata + 8, "strl", 4);
     /* generic header */
     memcpy (buffdata + 12, "strh", 4);
-    GST_WRITE_UINT32_LE (buffdata + 16, sizeof (gst_riff_strh));
+    GST_WRITE_UINT32_LE (buffdata + 16, sizeof (gst_riff_strh_full));
     /* the actual header */
     GST_WRITE_UINT32_LE (buffdata + 20, avipad->hdr.type);
     GST_WRITE_UINT32_LE (buffdata + 24, avipad->hdr.fcc_handler);
@@ -998,26 +1078,32 @@ gst_avi_mux_riff_get_avi_header (GstAviMux * avimux)
     GST_WRITE_UINT32_LE (buffdata + 56, avipad->hdr.bufsize);
     GST_WRITE_UINT32_LE (buffdata + 60, avipad->hdr.quality);
     GST_WRITE_UINT32_LE (buffdata + 64, avipad->hdr.samplesize);
+    GST_WRITE_UINT16_LE (buffdata + 68, 0);
+    GST_WRITE_UINT16_LE (buffdata + 70, 0);
+    GST_WRITE_UINT16_LE (buffdata + 72, 0);
+    GST_WRITE_UINT16_LE (buffdata + 74, 0);
+    buffdata += 76;
+    highmark += 76;
 
     if (avipad->is_video) {
       /* the video header */
-      memcpy (buffdata + 68, "strf", 4);
-      GST_WRITE_UINT32_LE (buffdata + 72,
+      memcpy (buffdata + 0, "strf", 4);
+      GST_WRITE_UINT32_LE (buffdata + 4,
           sizeof (gst_riff_strf_vids) + codec_size);
       /* the actual header */
-      GST_WRITE_UINT32_LE (buffdata + 76, vidpad->vids.size + codec_size);
-      GST_WRITE_UINT32_LE (buffdata + 80, vidpad->vids.width);
-      GST_WRITE_UINT32_LE (buffdata + 84, vidpad->vids.height);
-      GST_WRITE_UINT16_LE (buffdata + 88, vidpad->vids.planes);
-      GST_WRITE_UINT16_LE (buffdata + 90, vidpad->vids.bit_cnt);
-      GST_WRITE_UINT32_LE (buffdata + 92, vidpad->vids.compression);
-      GST_WRITE_UINT32_LE (buffdata + 96, vidpad->vids.image_size);
-      GST_WRITE_UINT32_LE (buffdata + 100, vidpad->vids.xpels_meter);
-      GST_WRITE_UINT32_LE (buffdata + 104, vidpad->vids.ypels_meter);
-      GST_WRITE_UINT32_LE (buffdata + 108, vidpad->vids.num_colors);
-      GST_WRITE_UINT32_LE (buffdata + 112, vidpad->vids.imp_colors);
-      buffdata += 116;
-      highmark += 116;
+      GST_WRITE_UINT32_LE (buffdata + 8, vidpad->vids.size + codec_size);
+      GST_WRITE_UINT32_LE (buffdata + 12, vidpad->vids.width);
+      GST_WRITE_UINT32_LE (buffdata + 16, vidpad->vids.height);
+      GST_WRITE_UINT16_LE (buffdata + 20, vidpad->vids.planes);
+      GST_WRITE_UINT16_LE (buffdata + 22, vidpad->vids.bit_cnt);
+      GST_WRITE_UINT32_LE (buffdata + 24, vidpad->vids.compression);
+      GST_WRITE_UINT32_LE (buffdata + 28, vidpad->vids.image_size);
+      GST_WRITE_UINT32_LE (buffdata + 32, vidpad->vids.xpels_meter);
+      GST_WRITE_UINT32_LE (buffdata + 36, vidpad->vids.ypels_meter);
+      GST_WRITE_UINT32_LE (buffdata + 40, vidpad->vids.num_colors);
+      GST_WRITE_UINT32_LE (buffdata + 44, vidpad->vids.imp_colors);
+      buffdata += 48;
+      highmark += 48;
 
       /* include codec data, if any */
       if (codec_size) {
@@ -1027,20 +1113,62 @@ gst_avi_mux_riff_get_avi_header (GstAviMux * avimux)
         buffdata += codec_size;
         highmark += codec_size;
       }
+      /* padding */
+      if (highmark & 0x1) {
+        highmark++;
+        buffdata++;
+      }
+
+      /* add video property data, mainly for aspect ratio, if any */
+      if (vprp_size) {
+        gint f;
+
+        /* the vprp header */
+        memcpy (buffdata + 0, "vprp", 4);
+        GST_WRITE_UINT32_LE (buffdata + 4, vprp_size);
+        /* the actual data */
+        GST_WRITE_UINT32_LE (buffdata + 8, vidpad->vprp.format_token);
+        GST_WRITE_UINT32_LE (buffdata + 12, vidpad->vprp.standard);
+        GST_WRITE_UINT32_LE (buffdata + 16, vidpad->vprp.vert_rate);
+        GST_WRITE_UINT32_LE (buffdata + 20, vidpad->vprp.hor_t_total);
+        GST_WRITE_UINT32_LE (buffdata + 24, vidpad->vprp.vert_lines);
+        GST_WRITE_UINT32_LE (buffdata + 28, vidpad->vprp.aspect);
+        GST_WRITE_UINT32_LE (buffdata + 32, vidpad->vprp.width);
+        GST_WRITE_UINT32_LE (buffdata + 36, vidpad->vprp.height);
+        GST_WRITE_UINT32_LE (buffdata + 40, vidpad->vprp.fields);
+        buffdata += 44;
+        highmark += 44;
+        for (f = 0; f < vidpad->vprp.fields; ++f) {
+          gst_riff_vprp_video_field_desc *fd;
+
+          fd = &(vidpad->vprp.field_info[f]);
+          GST_WRITE_UINT32_LE (buffdata + 0, fd->compressed_bm_height);
+          GST_WRITE_UINT32_LE (buffdata + 4, fd->compressed_bm_width);
+          GST_WRITE_UINT32_LE (buffdata + 8, fd->valid_bm_height);
+          GST_WRITE_UINT32_LE (buffdata + 12, fd->valid_bm_width);
+          GST_WRITE_UINT32_LE (buffdata + 16, fd->valid_bm_x_offset);
+          GST_WRITE_UINT32_LE (buffdata + 20, fd->valid_bm_y_offset);
+          GST_WRITE_UINT32_LE (buffdata + 24, fd->video_x_t_offset);
+          GST_WRITE_UINT32_LE (buffdata + 28, fd->video_y_start);
+          buffdata += 32;
+          highmark += 32;
+        }
+      }
     } else {
       /* the audio header */
-      memcpy (buffdata + 68, "strf", 4);
-      GST_WRITE_UINT32_LE (buffdata + 72,
-          sizeof (gst_riff_strf_auds) + codec_size);
+      memcpy (buffdata + 0, "strf", 4);
+      GST_WRITE_UINT32_LE (buffdata + 4,
+          sizeof (gst_riff_strf_auds) + 2 + codec_size);
       /* the actual header */
-      GST_WRITE_UINT16_LE (buffdata + 76, audpad->auds.format);
-      GST_WRITE_UINT16_LE (buffdata + 78, audpad->auds.channels);
-      GST_WRITE_UINT32_LE (buffdata + 80, audpad->auds.rate);
-      GST_WRITE_UINT32_LE (buffdata + 84, audpad->auds.av_bps);
-      GST_WRITE_UINT16_LE (buffdata + 88, audpad->auds.blockalign);
-      GST_WRITE_UINT16_LE (buffdata + 90, audpad->auds.size);
-      buffdata += 92;
-      highmark += 92;
+      GST_WRITE_UINT16_LE (buffdata + 8, audpad->auds.format);
+      GST_WRITE_UINT16_LE (buffdata + 10, audpad->auds.channels);
+      GST_WRITE_UINT32_LE (buffdata + 12, audpad->auds.rate);
+      GST_WRITE_UINT32_LE (buffdata + 16, audpad->auds.av_bps);
+      GST_WRITE_UINT16_LE (buffdata + 20, audpad->auds.blockalign);
+      GST_WRITE_UINT16_LE (buffdata + 22, audpad->auds.size);
+      GST_WRITE_UINT16_LE (buffdata + 24, codec_size);
+      buffdata += 26;
+      highmark += 26;
 
       /* include codec data, if any */
       if (codec_size) {
@@ -1050,6 +1178,11 @@ gst_avi_mux_riff_get_avi_header (GstAviMux * avimux)
         buffdata += codec_size;
         highmark += codec_size;
       }
+      /* padding */
+      if (highmark & 0x1) {
+        highmark++;
+        buffdata++;
+      }
     }
 
     /* odml superindex chunk */
@@ -1417,6 +1550,10 @@ gst_avi_mux_start_file (GstAviMux * avimux)
     }
   }
 
+  /* let downstream know we think in BYTES and expect to do seeking later on */
+  gst_pad_push_event (avimux->srcpad,
+      gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_BYTES, 0, -1, 0));
+
   /* header */
   avimux->avi_hdr.streams = g_slist_length (avimux->sinkpads);
   avimux->is_bigfile = FALSE;
