@@ -26,14 +26,21 @@
  *
  * <refsect2>
  * <para>
- * icydemux accepts data streams with ICY metadata at known intervals, as transmitted from an upstream element (usually read as response headers from an HTTP stream). The mime type of the data between the tag blocks is detected using typefind functions, and the appropriate output mime type set on outgoing buffers. 
+ * icydemux accepts data streams with ICY metadata at known intervals, as
+ * transmitted from an upstream element (usually read as response headers from
+ * an HTTP stream). The mime type of the data between the tag blocks is
+ * detected using typefind functions, and the appropriate output mime type set
+ * on outgoing buffers. 
  * </para>
  * <title>Example launch line</title>
  * <para>
  * <programlisting>
- * gst-launch gnomevfssrc location=http://some.server/ ! icydemux ! fakesink -t
+ * gst-launch souphttpsrc location=http://some.server/ iradio-mode=true ! icydemux ! fakesink -t
  * </programlisting>
- * This pipeline should read any available ICY tag information and output it. The contents of the stream should be detected, and the appropriate mime type set on buffers produced from icydemux.
+ * This pipeline should read any available ICY tag information and output it.
+ * The contents of the stream should be detected, and the appropriate mime
+ * type set on buffers produced from icydemux. (Using gnomevfssrc, neonhttpsrc
+ * or giosrc instead of souphttpsrc should also work.)
  * </para>
  * </refsect2>
  */
@@ -42,6 +49,7 @@
 #endif
 #include <gst/gst.h>
 #include <gst/gst-i18n-plugin.h>
+#include <gst/tag/tag.h>
 
 #include "gsticydemux.h"
 
@@ -285,40 +293,14 @@ gst_icydemux_remove_srcpad (GstICYDemux * icydemux)
   return res;
 };
 
-/* The following two charset mangling functions were copied from gnomevfssrc.
- * Preserve them under the unverified assumption that they do something vaguely
- * worthwhile.
- */
-static char *
-unicodify (const char *str, int len, ...)
+static gchar *
+gst_icydemux_unicodify (const gchar * str)
 {
-  char *ret = NULL, *cset;
-  va_list args;
-  gsize bytes_read, bytes_written;
+  const gchar *env_vars[] = { "GST_ICY_TAG_ENCODING",
+    "GST_TAG_ENCODING", NULL
+  };
 
-  if (g_utf8_validate (str, len, NULL))
-    return g_strndup (str, len >= 0 ? len : strlen (str));
-
-  va_start (args, len);
-  while ((cset = va_arg (args, char *)) != NULL)
-  {
-    if (!strcmp (cset, "locale"))
-      ret = g_locale_to_utf8 (str, len, &bytes_read, &bytes_written, NULL);
-    else
-      ret = g_convert (str, len, "UTF-8", cset,
-          &bytes_read, &bytes_written, NULL);
-    if (ret)
-      break;
-  }
-  va_end (args);
-
-  return ret;
-}
-
-static char *
-gst_icydemux_unicodify (const char *str)
-{
-  return unicodify (str, -1, "locale", "ISO-8859-1", NULL);
+  return gst_tag_freeform_string_to_utf8 (str, -1, env_vars);
 }
 
 static void
@@ -516,8 +498,9 @@ gst_icydemux_chain (GstPad * pad, GstBuffer * buf)
   GstFlowReturn ret = GST_FLOW_OK;
 
   icydemux = GST_ICYDEMUX (GST_PAD_PARENT (pad));
-  g_return_val_if_fail (GST_IS_ICYDEMUX (icydemux), GST_FLOW_ERROR);
-  g_return_val_if_fail (icydemux->meta_interval >= 0, GST_FLOW_ERROR);
+
+  if (G_UNLIKELY (icydemux->meta_interval < 0))
+    goto not_negotiated;
 
   if (icydemux->meta_interval == 0) {
     ret = gst_icydemux_typefind_or_forward (icydemux, buf);
@@ -577,6 +560,15 @@ done:
   gst_buffer_unref (buf);
 
   return ret;
+
+/* ERRORS */
+not_negotiated:
+  {
+    GST_WARNING_OBJECT (icydemux, "meta_interval not set, buffer probably had "
+        "no caps set. Try enabling iradio-mode on the http source element");
+    gst_buffer_unref (buf);
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
 }
 
 static GstStateChangeReturn
