@@ -21,22 +21,50 @@
 
 /**
  * SECTION:element-level
+ * @short_description: audio level analyzer
  *
  * <refsect2>
  * <para>
- * Level analyses incoming audio buffers and, if the
- * <link linkend="GstLevel--message">message property</link> is #TRUE,
- * generates an element message named
+ * Level analyses incoming audio buffers and, if the #GstLevel:message property
+ * is #TRUE, generates an element message named
  * <classname>&quot;level&quot;</classname>:
- * after each interval of time given by the
- * <link linkend="GstLevel--interval">interval property</link>.
+ * after each interval of time given by the #GstLevel:interval property.
  * The message's structure contains four fields:
  * <itemizedlist>
  * <listitem>
  *   <para>
  *   #GstClockTime
+ *   <classname>&quot;timestamp&quot;</classname>:
+ *   the timestamp of the buffer that triggered the message.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;stream-time&quot;</classname>:
+ *   the stream time of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;running-time&quot;</classname>:
+ *   the running_time of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;duration&quot;</classname>:
+ *   the duration of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
  *   <classname>&quot;endtime&quot;</classname>:
- *   the end time of the buffer that triggered the message
+ *   the end time of the buffer that triggered the message as stream time (this
+ *   is deprecated, as it can be calculated from stream-time + duration)
  *   </para>
  * </listitem>
  * <listitem>
@@ -66,7 +94,7 @@
  *   the Root Mean Square (or average power) level in dB for each channel
  *   </para>
  * </listitem>
-  * </itemizedlist>
+ * </itemizedlist>
  * </para>
  * <title>Example application</title>
  * <para>
@@ -82,6 +110,8 @@
 #include <math.h>
 #include <gst/gst.h>
 #include <gst/audio/audio.h>
+/*#include <liboil/liboil.h>*/
+
 #include "gstlevel.h"
 
 GST_DEBUG_CATEGORY_STATIC (level_debug);
@@ -100,8 +130,8 @@ static GstStaticPadTemplate sink_template_factory =
         "rate = (int) [ 1, MAX ], "
         "channels = (int) [ 1, MAX ], "
         "endianness = (int) BYTE_ORDER, "
-        "width = (int) { 8, 16 }, "
-        "depth = (int) { 8, 16 }, "
+        "width = (int) { 8, 16, 32 }, "
+        "depth = (int) { 8, 16, 32 }, "
         "signed = (boolean) true; "
         "audio/x-raw-float, "
         "rate = (int) [ 1, MAX ], "
@@ -117,8 +147,8 @@ static GstStaticPadTemplate src_template_factory =
         "rate = (int) [ 1, MAX ], "
         "channels = (int) [ 1, MAX ], "
         "endianness = (int) BYTE_ORDER, "
-        "width = (int) { 8, 16 }, "
-        "depth = (int) { 8, 16 }, "
+        "width = (int) { 8, 16, 32 }, "
+        "depth = (int) { 8, 16, 32 }, "
         "signed = (boolean) true; "
         "audio/x-raw-float, "
         "rate = (int) [ 1, MAX ], "
@@ -310,7 +340,7 @@ gst_level_get_property (GObject * object, guint prop_id,
  */
 
 #define DEFINE_INT_LEVEL_CALCULATOR(TYPE, RESOLUTION)                         \
-static void inline                                                          \
+static void inline                                                            \
 gst_level_calculate_##TYPE (gpointer data, guint num, guint channels,         \
                             gdouble *NCS, gdouble *NPS)                       \
 {                                                                             \
@@ -324,9 +354,9 @@ gst_level_calculate_##TYPE (gpointer data, guint num, guint channels,         \
   /* *NCS = 0.0; Normalized Cumulative Square */                              \
   /* *NPS = 0.0; Normalized Peask Square */                                   \
                                                                               \
-  normalizer = (gdouble) (1 << (RESOLUTION * 2));                             \
+  normalizer = (gdouble) (G_GINT64_CONSTANT(1) << (RESOLUTION * 2));          \
                                                                               \
-  /* oil_squaresum_f64(&squaresum,in,num); */                                 \
+  /* oil_squaresum_shifted_s16(&squaresum,in,num); */                         \
   for (j = 0; j < num; j += channels)                                         \
   {                                                                           \
     square = ((gdouble) in[j]) * in[j];                                       \
@@ -338,6 +368,7 @@ gst_level_calculate_##TYPE (gpointer data, guint num, guint channels,         \
   *NPS = peaksquare / normalizer;                                             \
 }
 
+DEFINE_INT_LEVEL_CALCULATOR (gint32, 31);
 DEFINE_INT_LEVEL_CALCULATOR (gint16, 15);
 DEFINE_INT_LEVEL_CALCULATOR (gint8, 7);
 
@@ -370,6 +401,17 @@ gst_level_calculate_##TYPE (gpointer data, guint num, guint channels,         \
 DEFINE_FLOAT_LEVEL_CALCULATOR (gfloat);
 DEFINE_FLOAT_LEVEL_CALCULATOR (gdouble);
 
+/* we would need stride to deinterleave also
+static void inline
+gst_level_calculate_gdouble (gpointer data, guint num, guint channels,
+                            gdouble *NCS, gdouble *NPS)
+{
+  oil_squaresum_f64(NCS,(gdouble *)data,num);
+  *NPS = 0.0;
+}
+*/
+
+
 static gint
 structure_get_int (GstStructure * structure, const gchar * field)
 {
@@ -406,6 +448,9 @@ gst_level_set_caps (GstBaseTransform * trans, GstCaps * in, GstCaps * out)
       case 16:
         filter->process = gst_level_calculate_gint16;
         break;
+      case 32:
+        filter->process = gst_level_calculate_gint32;
+        break;
     }
   } else if (strcmp (mimetype, "audio/x-raw-float") == 0) {
     GST_DEBUG_OBJECT (filter, "use float, %u", filter->width);
@@ -454,15 +499,29 @@ gst_level_start (GstBaseTransform * trans)
 }
 
 static GstMessage *
-gst_level_message_new (GstLevel * l, GstClockTime endtime)
+gst_level_message_new (GstLevel * level, GstClockTime timestamp,
+    GstClockTime duration)
 {
+  GstBaseTransform *trans = GST_BASE_TRANSFORM_CAST (level);
   GstStructure *s;
   GValue v = { 0, };
+  GstClockTime endtime, running_time, stream_time;
 
   g_value_init (&v, GST_TYPE_LIST);
 
-  s = gst_structure_new ("level", "endtime", GST_TYPE_CLOCK_TIME,
-      endtime, NULL);
+  running_time = gst_segment_to_running_time (&trans->segment, GST_FORMAT_TIME,
+      timestamp);
+  stream_time = gst_segment_to_stream_time (&trans->segment, GST_FORMAT_TIME,
+      timestamp);
+  /* endtime is for backwards compatibility */
+  endtime = stream_time + duration;
+
+  s = gst_structure_new ("level",
+      "endtime", GST_TYPE_CLOCK_TIME, endtime,
+      "timestamp", G_TYPE_UINT64, timestamp,
+      "stream-time", G_TYPE_UINT64, stream_time,
+      "running-time", G_TYPE_UINT64, running_time,
+      "duration", G_TYPE_UINT64, duration, NULL);
   /* will copy-by-value */
   gst_structure_set_value (s, "rms", &v);
   gst_structure_set_value (s, "peak", &v);
@@ -470,7 +529,7 @@ gst_level_message_new (GstLevel * l, GstClockTime endtime)
 
   g_value_unset (&v);
 
-  return gst_message_new_element (GST_OBJECT (l), s);
+  return gst_message_new_element (GST_OBJECT (level), s);
 }
 
 static void
@@ -584,6 +643,10 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
     }
   }
 
+  if (G_UNLIKELY (!filter->num_frames)) {
+    /* remember start timestamp for message */
+    filter->message_ts = GST_BUFFER_TIMESTAMP (in);
+  }
   filter->num_frames += num_frames;
 
   /* do we need to message ? */
@@ -591,16 +654,14 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
       GST_CLOCK_TIME_TO_FRAMES (filter->interval, filter->rate)) {
     if (filter->message) {
       GstMessage *m;
-      GstClockTime endtime =
-          gst_segment_to_stream_time (&trans->segment, GST_FORMAT_TIME,
-          GST_BUFFER_TIMESTAMP (in))
-          + GST_FRAMES_TO_CLOCK_TIME (num_frames, filter->rate);
+      GstClockTime duration =
+          GST_FRAMES_TO_CLOCK_TIME (filter->num_frames, filter->rate);
 
-      m = gst_level_message_new (filter, endtime);
+      m = gst_level_message_new (filter, filter->message_ts, duration);
 
       GST_LOG_OBJECT (filter,
-          "message: end time %" GST_TIME_FORMAT ", num_frames %d",
-          GST_TIME_ARGS (endtime), filter->num_frames);
+          "message: ts %" GST_TIME_FORMAT ", num_frames %d",
+          GST_TIME_ARGS (filter->message_ts), filter->num_frames);
 
       for (i = 0; i < filter->channels; ++i) {
         double RMS;
@@ -649,6 +710,8 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
+  /*oil_init (); */
+
   return gst_element_register (plugin, "level", GST_RANK_NONE, GST_TYPE_LEVEL);
 }
 
