@@ -36,8 +36,37 @@
  * <listitem>
  *   <para>
  *   #GstClockTime
+ *   <classname>&quot;timestamp&quot;</classname>:
+ *   the timestamp of the buffer that triggered the message.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;stream-time&quot;</classname>:
+ *   the stream time of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;running-time&quot;</classname>:
+ *   the running_time of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;duration&quot;</classname>:
+ *   the duration of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
  *   <classname>&quot;endtime&quot;</classname>:
- *   the end time of the buffer that triggered the message. Always present.
+ *   the end time of the buffer that triggered the message as stream time (this
+ *   is deprecated, as it can be calculated from stream-time + duration)
  *   </para>
  * </listitem>
  * <listitem>
@@ -123,10 +152,10 @@ GST_ELEMENT_DETAILS ("Spectrum analyzer",
     " channels = (int) [ 1, MAX ]"
 
 /* Spectrum properties */
-#define DEFAULT_SIGNAL_SPECTRUM		TRUE
-#define DEFAULT_SIGNAL_MAGNITUDE	TRUE
-#define DEFAULT_SIGNAL_PHASE		FALSE
-#define DEFAULT_SIGNAL_INTERVAL		(GST_SECOND / 10)
+#define DEFAULT_MESSAGE			TRUE
+#define DEFAULT_MESSAGE_MAGNITUDE	TRUE
+#define DEFAULT_MESSAGE_PHASE		FALSE
+#define DEFAULT_INTERVAL		(GST_SECOND / 10)
 #define DEFAULT_BANDS			128
 #define DEFAULT_THRESHOLD		-60
 
@@ -136,10 +165,10 @@ GST_ELEMENT_DETAILS ("Spectrum analyzer",
 enum
 {
   PROP_0,
-  PROP_SIGNAL_SPECTRUM,
-  PROP_SIGNAL_MAGNITUDE,
-  PROP_SIGNAL_PHASE,
-  PROP_SIGNAL_INTERVAL,
+  PROP_MESSAGE,
+  PROP_MESSAGE_MAGNITUDE,
+  PROP_MESSAGE_PHASE,
+  PROP_INTERVAL,
   PROP_BANDS,
   PROP_THRESHOLD
 };
@@ -200,28 +229,30 @@ gst_spectrum_class_init (GstSpectrumClass * klass)
 
   filter_class->setup = GST_DEBUG_FUNCPTR (gst_spectrum_setup);
 
-  g_object_class_install_property (gobject_class, PROP_SIGNAL_SPECTRUM,
+  g_object_class_install_property (gobject_class, PROP_MESSAGE,
       g_param_spec_boolean ("message", "Message",
-          "Post a level message for each passed interval",
-          DEFAULT_SIGNAL_SPECTRUM, G_PARAM_READWRITE));
+          "Whether to post a 'spectrum' element message on the bus for each "
+          "passed interval", DEFAULT_MESSAGE, G_PARAM_READWRITE));
 
-  g_object_class_install_property (gobject_class, PROP_SIGNAL_MAGNITUDE,
+  g_object_class_install_property (gobject_class, PROP_MESSAGE_MAGNITUDE,
       g_param_spec_boolean ("message-magnitude", "Magnitude",
-          "Post the magnitude of the spectrum",
-          DEFAULT_SIGNAL_MAGNITUDE, G_PARAM_READWRITE));
+          "Whether to add a 'magnitude' field to the structure of any "
+          "'spectrum' element messages posted on the bus",
+          DEFAULT_MESSAGE_MAGNITUDE, G_PARAM_READWRITE));
 
-  g_object_class_install_property (gobject_class, PROP_SIGNAL_PHASE,
+  g_object_class_install_property (gobject_class, PROP_MESSAGE_PHASE,
       g_param_spec_boolean ("message-phase", "Phase",
-          "Post the phase of the spectrum",
-          DEFAULT_SIGNAL_PHASE, G_PARAM_READWRITE));
+          "Whether to add a 'phase' field to the structure of any "
+          "'spectrum' element messages posted on the bus",
+          DEFAULT_MESSAGE_PHASE, G_PARAM_READWRITE));
 
-  g_object_class_install_property (gobject_class, PROP_SIGNAL_INTERVAL,
+  g_object_class_install_property (gobject_class, PROP_INTERVAL,
       g_param_spec_uint64 ("interval", "Interval",
           "Interval of time between message posts (in nanoseconds)",
-          1, G_MAXUINT64, DEFAULT_SIGNAL_INTERVAL, G_PARAM_READWRITE));
+          1, G_MAXUINT64, DEFAULT_INTERVAL, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_BANDS,
-      g_param_spec_uint ("bands", "Bands", "number of frequency bands",
+      g_param_spec_uint ("bands", "Bands", "Number of frequency bands",
           0, G_MAXUINT, DEFAULT_BANDS, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_THRESHOLD,
@@ -238,10 +269,10 @@ gst_spectrum_init (GstSpectrum * spectrum, GstSpectrumClass * g_class)
 {
   spectrum->adapter = gst_adapter_new ();
 
-  spectrum->message = DEFAULT_SIGNAL_SPECTRUM;
-  spectrum->message_magnitude = DEFAULT_SIGNAL_MAGNITUDE;
-  spectrum->message_phase = DEFAULT_SIGNAL_PHASE;
-  spectrum->interval = DEFAULT_SIGNAL_INTERVAL;
+  spectrum->message = DEFAULT_MESSAGE;
+  spectrum->message_magnitude = DEFAULT_MESSAGE_MAGNITUDE;
+  spectrum->message_phase = DEFAULT_MESSAGE_PHASE;
+  spectrum->interval = DEFAULT_INTERVAL;
   spectrum->bands = DEFAULT_BANDS;
   spectrum->threshold = DEFAULT_THRESHOLD;
 
@@ -292,16 +323,16 @@ gst_spectrum_set_property (GObject * object, guint prop_id,
   GstSpectrum *filter = GST_SPECTRUM (object);
 
   switch (prop_id) {
-    case PROP_SIGNAL_SPECTRUM:
+    case PROP_MESSAGE:
       filter->message = g_value_get_boolean (value);
       break;
-    case PROP_SIGNAL_MAGNITUDE:
+    case PROP_MESSAGE_MAGNITUDE:
       filter->message_magnitude = g_value_get_boolean (value);
       break;
-    case PROP_SIGNAL_PHASE:
+    case PROP_MESSAGE_PHASE:
       filter->message_phase = g_value_get_boolean (value);
       break;
-    case PROP_SIGNAL_INTERVAL:
+    case PROP_INTERVAL:
       filter->interval = g_value_get_uint64 (value);
       break;
     case PROP_BANDS:
@@ -345,16 +376,16 @@ gst_spectrum_get_property (GObject * object, guint prop_id,
   GstSpectrum *filter = GST_SPECTRUM (object);
 
   switch (prop_id) {
-    case PROP_SIGNAL_SPECTRUM:
+    case PROP_MESSAGE:
       g_value_set_boolean (value, filter->message);
       break;
-    case PROP_SIGNAL_MAGNITUDE:
+    case PROP_MESSAGE_MAGNITUDE:
       g_value_set_boolean (value, filter->message_magnitude);
       break;
-    case PROP_SIGNAL_PHASE:
+    case PROP_MESSAGE_PHASE:
       g_value_set_boolean (value, filter->message_phase);
       break;
-    case PROP_SIGNAL_INTERVAL:
+    case PROP_INTERVAL:
       g_value_set_uint64 (value, filter->interval);
       break;
     case PROP_BANDS:
@@ -447,22 +478,37 @@ gst_spectrum_setup (GstAudioFilter * base, GstRingBufferSpec * format)
 }
 
 static GstMessage *
-gst_spectrum_message_new (GstSpectrum * spectrum, GstClockTime endtime)
+gst_spectrum_message_new (GstSpectrum * spectrum, GstClockTime timestamp,
+    GstClockTime duration)
 {
+  GstBaseTransform *trans = GST_BASE_TRANSFORM_CAST (spectrum);
   GstStructure *s;
   GValue v = { 0, };
   GValue *l;
   guint i;
   gfloat *spect_magnitude = spectrum->spect_magnitude;
   gfloat *spect_phase = spectrum->spect_phase;
+  GstClockTime endtime, running_time, stream_time;
 
   GST_DEBUG_OBJECT (spectrum, "preparing message, spect = %p, bands =%d ",
       spect_magnitude, spectrum->bands);
 
-  s = gst_structure_new ("spectrum", "endtime", GST_TYPE_CLOCK_TIME,
-      endtime, NULL);
+  running_time = gst_segment_to_running_time (&trans->segment, GST_FORMAT_TIME,
+      timestamp);
+  stream_time = gst_segment_to_stream_time (&trans->segment, GST_FORMAT_TIME,
+      timestamp);
+  /* endtime is for backwards compatibility */
+  endtime = stream_time + duration;
+
+  s = gst_structure_new ("spectrum",
+      "endtime", GST_TYPE_CLOCK_TIME, endtime,
+      "timestamp", G_TYPE_UINT64, timestamp,
+      "stream-time", G_TYPE_UINT64, stream_time,
+      "running-time", G_TYPE_UINT64, running_time,
+      "duration", G_TYPE_UINT64, duration, NULL);
 
   if (spectrum->message_magnitude) {
+    /* FIXME 0.11: this should be an array, not a list */
     g_value_init (&v, GST_TYPE_LIST);
     /* will copy-by-value */
     gst_structure_set_value (s, "magnitude", &v);
@@ -478,6 +524,7 @@ gst_spectrum_message_new (GstSpectrum * spectrum, GstClockTime endtime)
   }
 
   if (spectrum->message_phase) {
+    /* FIXME 0.11: this should be an array, not a list */
     g_value_init (&v, GST_TYPE_LIST);
     /* will copy-by-value */
     gst_structure_set_value (s, "phase", &v);
@@ -579,11 +626,9 @@ process_f##width (GstSpectrum *spectrum, const g##type *samples) \
   \
   for (i = 0, j = 0; i < nfft; i++) { \
     /* convert to mono */ \
-    for (k = 0, acc = 0; k < channels; k++) \
+    for (k = 0, acc = 0.0; k < channels; k++) \
       acc += samples[j++]; \
     in[i] = (g##type) (acc / channels); \
-    if (abs (in[i]) > 1.0) \
-      g_assert_not_reached(); \
   } \
   \
   if (!spectrum->fft_ctx) { \
@@ -636,11 +681,6 @@ gst_spectrum_transform_ip (GstBaseTransform * trans, GstBuffer * in)
   gint width = GST_AUDIO_FILTER (spectrum)->format.width / 8;
   gint nfft = 2 * spectrum->bands - 2;
 
-  GstClockTime endtime =
-      gst_segment_to_running_time (&trans->segment, GST_FORMAT_TIME,
-      GST_BUFFER_TIMESTAMP (in));
-  GstClockTime blktime = GST_FRAMES_TO_CLOCK_TIME (nfft, rate);
-
   GST_LOG_OBJECT (spectrum, "input size: %d bytes", GST_BUFFER_SIZE (in));
 
   /* can we do this nicer? */
@@ -655,13 +695,18 @@ gst_spectrum_transform_ip (GstBaseTransform * trans, GstBuffer * in)
 
     spectrum->process (spectrum, samples);
 
+    if (G_UNLIKELY (!spectrum->num_frames)) {
+      /* remember start timestamp for message */
+      spectrum->message_ts = GST_BUFFER_TIMESTAMP (in);
+    }
     spectrum->num_frames += nfft;
-    endtime += blktime;
     /* do we need to message ? */
     if (spectrum->num_frames >=
         GST_CLOCK_TIME_TO_FRAMES (spectrum->interval, rate)) {
       if (spectrum->message) {
         GstMessage *m;
+        GstClockTime duration =
+            GST_FRAMES_TO_CLOCK_TIME (spectrum->num_frames, rate);
 
         /* Calculate average */
         for (i = 0; i < spectrum->bands; i++) {
@@ -669,7 +714,7 @@ gst_spectrum_transform_ip (GstBaseTransform * trans, GstBuffer * in)
           spect_phase[i] /= spectrum->num_fft;
         }
 
-        m = gst_spectrum_message_new (spectrum, endtime);
+        m = gst_spectrum_message_new (spectrum, spectrum->message_ts, duration);
 
         gst_element_post_message (GST_ELEMENT (spectrum), m);
       }
