@@ -67,6 +67,7 @@ static GstBufferClass *v4l2buffer_parent_class = NULL;
 static gboolean
 gst_v4l2src_get_nearest_size (GstV4l2Src * v4l2src, guint32 pixelformat,
     gint * width, gint * height);
+static void gst_v4l2_buffer_pool_destroy (GstV4l2BufferPool * pool);
 
 static void
 gst_v4l2_buffer_finalize (GstV4l2Buffer * buffer)
@@ -107,17 +108,11 @@ gst_v4l2_buffer_finalize (GstV4l2Buffer * buffer)
     gst_mini_object_unref (GST_MINI_OBJECT (pool));
     munmap ((void *) GST_BUFFER_DATA (buffer), buffer->vbuffer.length);
 
-    GST_MINI_OBJECT_CLASS (v4l2buffer_parent_class)->
-        finalize (GST_MINI_OBJECT (buffer));
+    GST_MINI_OBJECT_CLASS (v4l2buffer_parent_class)->finalize (GST_MINI_OBJECT
+        (buffer));
   }
 }
 
-static void
-gst_v4l2_buffer_init (GstV4l2Buffer * xvimage, gpointer g_class)
-{
-  /* NOP */
-}
-
 static void
 gst_v4l2_buffer_class_init (gpointer g_class, gpointer class_data)
 {
@@ -144,7 +139,7 @@ gst_v4l2_buffer_get_type (void)
       NULL,
       sizeof (GstV4l2Buffer),
       0,
-      (GInstanceInitFunc) gst_v4l2_buffer_init,
+      NULL,
       NULL
     };
     _gst_v4l2_buffer_type = g_type_register_static (GST_TYPE_BUFFER,
@@ -163,9 +158,8 @@ gst_v4l2_buffer_new (GstV4l2BufferPool * pool, guint index, GstCaps * caps)
 
   GST_LOG ("creating buffer %u, %p in pool %p", index, ret, pool);
 
-  ret->pool = pool;
-  gst_mini_object_ref (GST_MINI_OBJECT (pool));
-  memset (&ret->vbuffer, 0x00, sizeof (ret->vbuffer));
+  ret->pool =
+      (GstV4l2BufferPool *) gst_mini_object_ref (GST_MINI_OBJECT (pool));
 
   ret->vbuffer.index = index;
   ret->vbuffer.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
@@ -237,11 +231,13 @@ gst_v4l2_buffer_pool_finalize (GstV4l2BufferPool * pool)
   if (pool->video_fd >= 0)
     close (pool->video_fd);
 
-  if (pool->buffers)
+  if (pool->buffers) {
     g_free (pool->buffers);
-  pool->buffers = NULL;
-  GST_MINI_OBJECT_CLASS (buffer_pool_parent_class)->
-      finalize (GST_MINI_OBJECT (pool));
+    pool->buffers = NULL;
+  }
+
+  GST_MINI_OBJECT_CLASS (buffer_pool_parent_class)->finalize (GST_MINI_OBJECT
+      (pool));
 }
 
 static void
@@ -326,7 +322,7 @@ buffer_new_failed:
   {
     gint errnosave = errno;
 
-    gst_mini_object_unref (GST_MINI_OBJECT (pool));
+    gst_v4l2_buffer_pool_destroy (pool);
 
     errno = errnosave;
 
@@ -379,10 +375,14 @@ gst_v4l2_buffer_pool_destroy (GstV4l2BufferPool * pool)
   pool->running = FALSE;
   g_mutex_unlock (pool->lock);
 
+  GST_DEBUG ("destroy pool");
+
   /* after this point, no more buffers will be queued or dequeued; no buffer
    * from pool->buffers that is NULL will be set to a buffer, and no buffer that
    * is not NULL will be pushed out. */
 
+  /* miniobjects have no dispose, so they can't break ref-cycles, as buffers ref
+   * the pool, we need to unref the buffer to properly finalize te pool */
   for (n = 0; n < pool->buffer_count; n++) {
     GstBuffer *buf;
 
@@ -406,7 +406,9 @@ gst_v4l2_buffer_pool_destroy (GstV4l2BufferPool * pool)
 #define YUV_ODD_BASE_RANK   50
 #define RGB_ODD_BASE_RANK   25
 #define BAYER_BASE_RANK     15
+#define S910_BASE_RANK      10
 #define GREY_BASE_RANK       5
+#define PWC_BASE_RANK        1
 
 static gint
 gst_v4l2src_format_get_rank (guint32 fourcc)
@@ -470,6 +472,14 @@ gst_v4l2src_format_get_rank (guint32 fourcc)
     case V4L2_PIX_FMT_SBGGR8:
       return BAYER_BASE_RANK;
 
+    case V4L2_PIX_FMT_SN9C10X:
+      return S910_BASE_RANK;
+
+    case V4L2_PIX_FMT_PWC1:
+      return PWC_BASE_RANK;
+    case V4L2_PIX_FMT_PWC2:
+      return PWC_BASE_RANK;
+
     default:
       break;
   }
@@ -690,11 +700,9 @@ gst_v4l2src_probe_caps_for_format_and_size (GstV4l2Src * v4l2src,
       }
     }
     if (!added) {
-      /* no range was added, make a default range */
-      GST_WARNING_OBJECT (v4l2src, "no range added, setting 0/1 to 100/1");
+      /* no range was added, leave the default range from the template */
+      GST_WARNING_OBJECT (v4l2src, "no range added, leaving default");
       g_value_unset (&rates);
-      g_value_init (&rates, GST_TYPE_FRACTION_RANGE);
-      gst_value_set_fraction_range_full (&rates, 0, 1, 100, 1);
     }
   } else if (ival.type == V4L2_FRMIVAL_TYPE_CONTINUOUS) {
     guint32 maxnum, maxdenom;
@@ -723,25 +731,30 @@ gst_v4l2src_probe_caps_for_format_and_size (GstV4l2Src * v4l2src,
     goto unknown_type;
   }
 
+return_data:
   s = gst_structure_copy (template);
   gst_structure_set (s, "width", G_TYPE_INT, (gint) width,
       "height", G_TYPE_INT, (gint) height, NULL);
-  gst_structure_set_value (s, "framerate", &rates);
-  g_value_unset (&rates);
 
+  if (G_IS_VALUE (&rates)) {
+    /* only change the framerate on the template when we have a valid probed new
+     * value */
+    gst_structure_set_value (s, "framerate", &rates);
+    g_value_unset (&rates);
+  }
   return s;
 
   /* ERRORS */
 enum_frameintervals_failed:
   {
     GST_DEBUG_OBJECT (v4l2src,
-        "Failed to enumerate frame sizes for %" GST_FOURCC_FORMAT "@%ux%u",
+        "Unable to enumerate intervals for %" GST_FOURCC_FORMAT "@%ux%u",
         GST_FOURCC_ARGS (pixelformat), width, height);
-    return NULL;
+    goto return_data;
   }
 unknown_type:
   {
-    /* I don't see how this is actually an error */
+    /* I don't see how this is actually an error, we ignore the format then */
     GST_WARNING_OBJECT (v4l2src,
         "Unknown frame interval type at %" GST_FOURCC_FORMAT "@%ux%u: %u",
         GST_FOURCC_ARGS (pixelformat), width, height, ival.type);
@@ -754,7 +767,7 @@ GstCaps *
 gst_v4l2src_probe_caps_for_format (GstV4l2Src * v4l2src, guint32 pixelformat,
     const GstStructure * template)
 {
-  GstCaps *ret;
+  GstCaps *ret = gst_caps_new_empty ();
   GstStructure *tmp;
 
 #ifdef VIDIOC_ENUM_FRAMESIZES
@@ -763,17 +776,20 @@ gst_v4l2src_probe_caps_for_format (GstV4l2Src * v4l2src, guint32 pixelformat,
   GList *results = NULL;
   guint32 w, h;
 
-  ret = gst_caps_new_empty ();
-
   memset (&size, 0, sizeof (struct v4l2_frmsizeenum));
   size.index = 0;
   size.pixel_format = pixelformat;
 
+  GST_DEBUG_OBJECT (v4l2src, "Enumerating frame sizes");
+
   if (ioctl (fd, VIDIOC_ENUM_FRAMESIZES, &size) < 0)
     goto enum_framesizes_failed;
 
   if (size.type == V4L2_FRMSIZE_TYPE_DISCRETE) {
     do {
+      GST_LOG_OBJECT (v4l2src, "got discrete frame size %dx%d",
+          size.discrete.width, size.discrete.height);
+
       w = MIN (size.discrete.width, G_MAXINT);
       h = MIN (size.discrete.height, G_MAXINT);
 
@@ -787,7 +803,16 @@ gst_v4l2src_probe_caps_for_format (GstV4l2Src * v4l2src, guint32 pixelformat,
 
       size.index++;
     } while (ioctl (fd, VIDIOC_ENUM_FRAMESIZES, &size) >= 0);
+    GST_DEBUG_OBJECT (v4l2src, "done iterating discrete frame sizes");
   } else if (size.type == V4L2_FRMSIZE_TYPE_STEPWISE) {
+    GST_DEBUG_OBJECT (v4l2src, "we have stepwise frame sizes:");
+    GST_DEBUG_OBJECT (v4l2src, "min width:   %d", size.stepwise.min_width);
+    GST_DEBUG_OBJECT (v4l2src, "min height:  %d", size.stepwise.min_height);
+    GST_DEBUG_OBJECT (v4l2src, "max width:   %d", size.stepwise.max_width);
+    GST_DEBUG_OBJECT (v4l2src, "min height:  %d", size.stepwise.max_height);
+    GST_DEBUG_OBJECT (v4l2src, "step width:  %d", size.stepwise.step_width);
+    GST_DEBUG_OBJECT (v4l2src, "step height: %d", size.stepwise.step_height);
+
     for (w = size.stepwise.min_width, h = size.stepwise.min_height;
         w < size.stepwise.max_width && h < size.stepwise.max_height;
         w += size.stepwise.step_width, h += size.stepwise.step_height) {
@@ -802,9 +827,16 @@ gst_v4l2src_probe_caps_for_format (GstV4l2Src * v4l2src, guint32 pixelformat,
       if (tmp)
         results = g_list_prepend (results, tmp);
     }
+    GST_DEBUG_OBJECT (v4l2src, "done iterating stepwise frame sizes");
   } else if (size.type == V4L2_FRMSIZE_TYPE_CONTINUOUS) {
     guint32 maxw, maxh;
 
+    GST_DEBUG_OBJECT (v4l2src, "we have continuous frame sizes:");
+    GST_DEBUG_OBJECT (v4l2src, "min width:   %d", size.stepwise.min_width);
+    GST_DEBUG_OBJECT (v4l2src, "min height:  %d", size.stepwise.min_height);
+    GST_DEBUG_OBJECT (v4l2src, "max width:   %d", size.stepwise.max_width);
+    GST_DEBUG_OBJECT (v4l2src, "min height:  %d", size.stepwise.max_height);
+
     w = MAX (size.stepwise.min_width, 1);
     h = MAX (size.stepwise.min_height, 1);
     maxw = MIN (size.stepwise.max_width, G_MAXINT);
@@ -868,7 +900,7 @@ unknown_type:
 default_frame_sizes:
 #endif /* defined VIDIOC_ENUM_FRAMESIZES */
   {
-    gint min_w, max_w, min_h, max_h;
+    gint min_w, max_w, min_h, max_h, fix_num = 0, fix_denom = 0;
 
     /* This code is for Linux < 2.6.19 */
     min_w = min_h = 1;
@@ -884,22 +916,47 @@ default_frame_sizes:
           GST_FOURCC_FORMAT, GST_FOURCC_ARGS (pixelformat));
     }
 
-    ret = gst_caps_new_empty ();
+    /* Since we can't get framerate directly, try to use the current norm */
+    if (v4l2src->v4l2object->norm && v4l2src->v4l2object->norms) {
+      GList *norms;
+      GstTunerNorm *norm;
+
+      for (norms = v4l2src->v4l2object->norms; norms != NULL;
+          norms = norms->next) {
+        norm = (GstTunerNorm *) norms->data;
+        if (!strcmp (norm->label, v4l2src->v4l2object->norm))
+          break;
+      }
+      /* If it's possible, set framerate to that (discrete) value */
+      if (norm) {
+        fix_num = gst_value_get_fraction_numerator (&norm->framerate);
+        fix_denom = gst_value_get_fraction_denominator (&norm->framerate);
+      }
+    }
+
     tmp = gst_structure_copy (template);
-    gst_structure_set (tmp,
-        "width", GST_TYPE_INT_RANGE, min_w, max_w,
-        "height", GST_TYPE_INT_RANGE, min_h, max_h,
-        "framerate", GST_TYPE_FRACTION_RANGE, (gint) 0, (gint) 1, (gint) 100,
-        (gint) 1, NULL);
+    if (fix_num) {
+      gst_structure_set (tmp,
+          "width", GST_TYPE_INT_RANGE, min_w, max_w,
+          "height", GST_TYPE_INT_RANGE, min_h, max_h,
+          "framerate", GST_TYPE_FRACTION, fix_num, fix_denom, NULL);
+    } else {
+      /* if norm can't be used, copy the template framerate */
+      gst_structure_set (tmp,
+          "width", GST_TYPE_INT_RANGE, min_w, max_w,
+          "height", GST_TYPE_INT_RANGE, min_h, max_h,
+          "framerate", GST_TYPE_FRACTION_RANGE, 0, 1, 100, 1, NULL);
+    }
     gst_caps_append_structure (ret, tmp);
+
+    return ret;
   }
-  return ret;
 }
 
 /******************************************************
  * gst_v4l2src_grab_frame ():
  *   grab a frame for capturing
- * return value: The captured frame number or -1 on error.
+ * return value: GST_FLOW_OK or GST_FLOW_ERROR
  ******************************************************/
 GstFlowReturn
 gst_v4l2src_grab_frame (GstV4l2Src * v4l2src, GstBuffer ** buf)
@@ -1106,19 +1163,10 @@ qbuf_failed:
 */
 }
 
-static gboolean
-fractions_are_equal (gint num1, gint den1, gint num2, gint den2)
-{
-  GValue fraction1 = { 0, }, fraction2 = {
-  0,};
-
-  g_value_init (&fraction1, GST_TYPE_FRACTION);
-  g_value_init (&fraction2, GST_TYPE_FRACTION);
-  gst_value_set_fraction (&fraction1, num1, den1);
-  gst_value_set_fraction (&fraction2, num2, den2);
-  /* we know we don't have to unset the values in this case */
-  return (gst_value_compare (&fraction1, &fraction2) == GST_VALUE_EQUAL);
-}
+/* Note about fraction simplification
+ * n1/d1 == n2/d2  is also written as  n1 == ( n2 * d1 ) / d2
+ */
+#define fractions_are_equal(n1,d1,n2,d2) ((n1) == gst_util_uint64_scale_int((n2), (d1), (d2)))
 
 /******************************************************
  * gst_v4l2src_set_capture():
@@ -1149,11 +1197,23 @@ gst_v4l2src_set_capture (GstV4l2Src * v4l2src, guint32 pixelformat,
   format.fmt.pix.width = width;
   format.fmt.pix.height = height;
   format.fmt.pix.pixelformat = pixelformat;
-  /* request whole frames; change when gstreamer supports interlaced video */
+  /* request whole frames; change when gstreamer supports interlaced video
+   * (INTERLACED mode returns frames where the fields have already been
+   *  combined, there are other modes for requesting fields individually) */
   format.fmt.pix.field = V4L2_FIELD_INTERLACED;
 
-  if (ioctl (fd, VIDIOC_S_FMT, &format) < 0)
-    goto set_fmt_failed;
+  if (ioctl (fd, VIDIOC_S_FMT, &format) < 0) {
+    if (errno != EINVAL)
+      goto set_fmt_failed;
+
+    /* try again with progressive video */
+    format.fmt.pix.width = width;
+    format.fmt.pix.height = height;
+    format.fmt.pix.pixelformat = pixelformat;
+    format.fmt.pix.field = V4L2_FIELD_NONE;
+    if (ioctl (fd, VIDIOC_S_FMT, &format) < 0)
+      goto set_fmt_failed;
+  }
 
   if (format.fmt.pix.width != width || format.fmt.pix.height != height)
     goto invalid_dimensions;
@@ -1176,7 +1236,9 @@ gst_v4l2src_set_capture (GstV4l2Src * v4l2src, guint32 pixelformat,
   /* Note: V4L2 provides the frame interval, we have the frame rate */
   if (fractions_are_equal (stream.parm.capture.timeperframe.numerator,
           stream.parm.capture.timeperframe.denominator, fps_d, fps_n)) {
-    GST_LOG_OBJECT (v4l2src, "Desired framerate already set, nothing to do");
+    GST_LOG_OBJECT (v4l2src, "Desired framerate already set");
+    v4l2src->fps_n = fps_n;
+    v4l2src->fps_d = fps_d;
     goto done;
   }
 
@@ -1473,6 +1535,7 @@ gst_v4l2src_get_nearest_size (GstV4l2Src * v4l2src, guint32 pixelformat,
 {
   struct v4l2_format fmt;
   int fd;
+  int r;
 
   g_return_val_if_fail (width != NULL, FALSE);
   g_return_val_if_fail (height != NULL, FALSE);
@@ -1491,7 +1554,17 @@ gst_v4l2src_get_nearest_size (GstV4l2Src * v4l2src, guint32 pixelformat,
   fmt.fmt.pix.pixelformat = pixelformat;
   fmt.fmt.pix.field = V4L2_FIELD_INTERLACED;
 
-  if (ioctl (fd, VIDIOC_TRY_FMT, &fmt) < 0) {
+  r = ioctl (fd, VIDIOC_TRY_FMT, &fmt);
+  if (r < 0 && errno == EINVAL) {
+    /* try again with progressive video */
+    fmt.fmt.pix.width = *width;
+    fmt.fmt.pix.height = *height;
+    fmt.fmt.pix.pixelformat = pixelformat;
+    fmt.fmt.pix.field = V4L2_FIELD_NONE;
+    r = ioctl (fd, VIDIOC_TRY_FMT, &fmt);
+  }
+
+  if (r < 0) {
     /* The driver might not implement TRY_FMT, in which case we will try
        S_FMT to probe */
     if (errno != ENOTTY)
@@ -1508,7 +1581,17 @@ gst_v4l2src_get_nearest_size (GstV4l2Src * v4l2src, guint32 pixelformat,
     fmt.fmt.pix.width = *width;
     fmt.fmt.pix.height = *height;
 
-    if (ioctl (fd, VIDIOC_S_FMT, &fmt) < 0)
+    r = ioctl (fd, VIDIOC_S_FMT, &fmt);
+    if (r < 0 && errno == EINVAL) {
+      /* try again with progressive video */
+      fmt.fmt.pix.width = *width;
+      fmt.fmt.pix.height = *height;
+      fmt.fmt.pix.pixelformat = pixelformat;
+      fmt.fmt.pix.field = V4L2_FIELD_NONE;
+      r = ioctl (fd, VIDIOC_S_FMT, &fmt);
+    }
+
+    if (r < 0)
       return FALSE;
   }
 
